import argparse
import time
from typing import Any, Dict, List, Literal, Optional, Union

import torch
import uvicorn
from fastapi import FastAPI, HTTPException
from pydantic import BaseModel, Field

class ChatCompletionRequest(BaseModel):
    model: str
    messages: Union[str, List[Dict[str, Any]]] = Field(examples=[[{"role": "user", "content": "hi"}]])
    temperature: Optional[float] = 0.7
    top_p: Optional[float] = 1.0
    max_tokens: Optional[int] = Field(default=512, examples=[None])

def create_app(args):
    if args.model_hub in ["ms", "modelscope"]:
        from modelscope import AutoModelForCausalLM, AutoTokenizer
    else:
        from transformers import AutoModelForCausalLM, AutoTokenizer

    app = FastAPI()
    model = AutoModelForCausalLM.from_pretrained(args.model, torch_dtype="auto").to(args.device)
    tokenizer = AutoTokenizer.from_pretrained(args.model)

    @app.post("/chat/completions")
    async def chat(request: ChatCompletionRequest):
        text = tokenizer.apply_chat_template(request.messages, tokenize=False, add_generation_prompt=True)
        model_inputs = tokenizer([text], return_tensors="pt").to(args.device)
        with torch.no_grad():
            generated_ids = model.generate(model_inputs.input_ids, max_new_tokens=request.max_tokens, temperature=request.temperature, top_p=request.top_p)
        generated_ids = [output_ids[len(input_ids) :] for input_ids, output_ids in zip(model_inputs.input_ids, generated_ids)]
        response = tokenizer.batch_decode(generated_ids, skip_special_tokens=True)[0]
        return {"response": response}

    return app


def main():
    parser = argparse.ArgumentParser(description="fastspeed: A fastapi deployment tool based on accelerate.")
    parser.add_argument("--model", type=str, required=True, help="Model id or model path.")
    parser.add_argument("--host", type=str, default="0.0.0.0", help="Host of API.")
    parser.add_argument("--port", type=int, default=8000, help="Port of API.")
    parser.add_argument("--model_hub", type=str, default="ms", choices=["hf", "huggingface", "ms", "modelscope"], help="Model hub.")
    parser.add_argument("--device", type=str, default="cuda", help="Device")

    args, unknown = parser.parse_known_args()
    app = create_app(args)
    uvicorn.run(app, host=args.host, port=args.port)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import io
import re
from pathlib import Path

from setuptools import find_packages, setup


def get_version(package):
    """
    Return package version from `__init__.py`.
    """
    version = Path(package, "__init__.py").read_text()
    return re.search("__version__ = ['']([^'']+)['']", version).group(1)


def get_long_description():
    """
    Return the README.
    """
    long_description = ""
    with open("README.md", encoding="utf8") as f:
        long_description += f.read()
    long_description += "\n\n"
    with open("CHANGELOG.md", encoding="utf8") as f:
        long_description += f.read()
    return long_description


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [str(path.parent) for path in Path(package).glob("**/__init__.py")]


def read_file(filename):
    with io.open(filename) as fp:
        return fp.read().strip()


def read_requirements(filename):
    return [line.strip() for line in read_file(filename).splitlines() if not line.startswith("#")]


setup(
    name="fastspeed",
    python_requires=">=3.6",
    version=get_version("fastspeed"),
    url="https://github.com/StudyExchange/fastspeed.git",
    project_urls={
        "Changelog": "https://github.com/StudyExchange/fastspeed.git/blob/master/CHANGELOG.md",
        "Source": "https://github.com/StudyExchange/fastspeed.git",
    },
    license="MIT",
    description="Fastspeed is a toolkit for serving LLM.",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    author="StudyExchange",
    author_email="StudyExchange@163.com",
    packages=find_packages(),
    entry_points={
        "console_scripts": [
            "fastspeed=fastspeed.main:main",
        ],
    },
    setup_requires=read_requirements("requirements-setup.txt"),
    install_requires=read_requirements("requirements-install.txt"),
    zip_safe=False,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3 :: Only",
    ],
)
